from .common import Color


class CanvasBackend:
    def __init__(self):
        self.fill = True
        self.stroke = True
        self.back_color = Color('white')
        self.fill_color = Color('blue')
        self.stroke_color = Color('black')
        self.size = (500, 500)
        self.frame = 0.1

    def init(self):
        raise NotImplementedError

    def start(self, main_func, mod):
        raise NotImplementedError

    def clear(self):
        pass

    def set_fill(self, yes):
        self.fill = yes

    def set_frame(self, l):
        self.frame = l

    def set_stroke(self, yes):
        self.stroke = yes

    def get_mouse_state(self):
        pass

    def get_keyboard_state(self):
        pass

    def set_size(self, w, h):
        self.size = (w, h)

    def set_background(self, color):
        self.back_color = color

    def draw_shape(self, shape):
        pass

    def draw_point(self, x, y):
        pass

    def draw_line(self, x1, y1, x2, y2):
        pass

    def draw_rectangle(self, x, y, w, h):
        pass

    def draw_ellipse(self, x, y, a, b):
        pass

    def draw_text(self, x, y, text, **kwargs):
        pass



from .backend_base import CanvasBackend
import tkinter as tk


class Backend(CanvasBackend):
    def __init__(self):
        CanvasBackend.__init__(self)
        self.win = tk.Tk()
        self.can = tk.Canvas(self.win)
        self.user_loop = None

    def init(self):
        self.can.configure(
            width=self.size[0],
            height=self.size[1],
            background=self.back_color.hashtag()
        )
        self.can.pack()

    def start(self, setup, loop):
        setup()
        self.user_loop = loop
        self.win.after(int(1000*self.frame), self.loop)
        self.win.mainloop()

    def loop(self):
        self.user_loop()
        self.win.after(int(1000*self.frame), self.loop)

    def clear(self):
        self.can.delete('all')

    def set_fill(self, yes):
        self.fill = yes

    def set_stroke(self, yes):
        self.stroke = yes

    def get_mouse_state(self):
        pass

    def get_keyboard_state(self):
        pass

    def set_size(self, w, h):
        self.size = (w, h)
        self.can.configure(width=w, height=h)

    def set_background(self, color):
        self.back_color = color

    def draw_shape(self, shape):
        pass

    def draw_point(self, x, y):
        fill = self.stroke_color.hashtag() if self.stroke else ''
        self.can.create_line(x, y, x, y, fill=fill)

    def draw_line(self, x1, y1, x2, y2):
        fill = self.stroke_color.hashtag() if self.stroke else ''
        self.can.create_arc(x1, y1, x2, y2, fill=fill)

    def draw_rectangle(self, x, y, w, h):
        fill = self.fill_color.hashtag() if self.fill else ''
        outline = self.stroke_color.hashtag() if self.stroke else ''
        self.can.create_rectangle(x, y, x+w, y+h, fill=fill, outline=outline)

    def draw_ellipse(self, x, y, a, b):
        x0 = x - a/2
        x1 = x + a/2
        y0 = y - b/2
        y1 = y + b/2
        fill = self.fill_color.hashtag() if self.fill else ''
        outline = self.stroke_color.hashtag() if self.stroke else ''
        self.can.create_oval(x0, y0, x1, y1, fill=fill, outline=outline)

    def draw_text(self, x, y, text, **kwargs):
        pass



color_tab = {
    'black': (0, 0, 0),
    'white': (1, 1, 1),
    'red': (1, 0, 0),
    'green': (0, 1, 0),
    'blue': (0, 0, 1),
    'grey': (0.5, 0.5, 0.5),
}


class Color:
    def __init__(self, col):
        self.r = 0.0
        self.g = 0.0
        self.b = 0.0
        if isinstance(col, str):
            self.r, self.g, self.b = color_tab[col]
        elif isinstance(col, Color):
            self.r = col.r
            self.g = col.g
            self.b = col.b
        elif isinstance(col, tuple):
            self.r = col[0]
            self.g = col[1]
            self.b = col[2]
        elif isinstance(col, float):
            self.r = col
            self.g = col
            self.b = col

    def hashtag(self):
        r = hex(int(255*self.r))[2:]
        g = hex(int(255*self.g))[2:]
        b = hex(int(255*self.b))[2:]
        if len(r) != 2:
            r = '0' + r
        if len(g) != 2:
            g = '0' + g
        if len(b) != 2:
            b = '0' + b
        return '#{}{}{}'.format(r, g, b)
